@@ -21,9 +21,9 @@ from grokcore.viewlet import *
 from grokcore.formlib import *
 
 from five.grok.components import Model, Container, Site, LocalUtility
-from five.grok.components import View, Form, AddForm
+from five.grok.components import Form, AddForm
 from five.grok.components import EditForm, DisplayForm
-from five.grok.components import ViewletManager, Viewlet
+from five.grok.components import View, ViewletManager
 from five.grok.formlib import AutoFields
 
 # backwards compatibility. Probably not needed by many, but just in case.
